<?php

namespace app\mobile\logic;

use app\model\AccountModel;
use app\model\AccountTokenModel;
use app\model\CommonModel;
use app\model\GroupModel;
use app\model\ThemeModel;
use app\model\VideoModel;
use think\Exception;
use think\facade\Config;
use think\facade\Db;
use think\response\Json;

class AccountLogic extends BaseLogic
{
    //登录
    public static function login(array $data = []): Json
    {
        Db::startTrans();
        try {

            $rs = AccountModel::field('id,username,name,password,salt,starttime,expiretime,status')
                ->where(['is_del' => CommonModel::$del_normal, 'username' => $data['username']])
                ->findOrEmpty();

            if ($rs->isEmpty()) throw new Exception('该卡号不存在，请仔细核对');
            if (getPassword($data['password'], $rs->salt) != $rs->password) throw new Exception('密码错误');

            $date = date('Y-m-d H:i:s');

            if (($date < $rs->starttime) || ($date > $rs->expiretime)) throw new Exception('该卡不在有效期内');


            $update_data = ['updaterid' => $rs->id, 'updater' => $rs->name, 'updatetime' => $date];

            if ($rs->status == AccountModel::$status_not_active) {
                //处理激活信息
                $update_data['status'] = AccountModel::$status_activated;
                $update_data['activetime'] = $date;
            }

            //根棍账户相关信息
            AccountModel::where(['is_del' => CommonModel::$del_normal, 'id' => $rs->id])->save($update_data);

            //维护token
            $token = base64_encode($rs->username . $rs->salt . (string)time());

            $res = AccountTokenModel::field('id')
                ->where('accountid', $rs->id)
                ->findOrEmpty()
                ->isEmpty();

            $expire = Config::get('common.expire');
            if ($res) AccountTokenModel::create(['token' => $token, 'expiretime' => date('Y-m-d H:i:s', time() + $expire), 'accountid' => $rs->id]);
            else AccountTokenModel::where(['accountid' => $rs->id])->update(['token' => $token, 'expiretime' => date('Y-m-d H:i:s', time() + $expire)]);

            Db::commit();

            return json_show(CommonModel::$success, '登录成功', ['token' => $token]);

        } catch (Exception $exception) {
            Db::rollback();
            return json_show(CommonModel::$error_param, $exception->getMessage());
        }

    }

    //登出
    public static function logout(): Json
    {

        $info = AccountTokenModel::where(['accountid' => self::$aid])->save(['token' => '', 'expiretime' => date('Y-m-d H:i:s')]);

        return $info ? json_show(CommonModel::$success, '登出成功') : json_show(CommonModel::$error_param, '登出失败');

    }

    //详情
    public static function info(): Json
    {

        $info = AccountModel::where(['id' => self::$aid])
            ->field('id,username,mobile,name,starttime,expiretime')
            ->findOrEmpty()
            ->toArray();

        return $info ? json_show(CommonModel::$success, '获取账户详情成功', $info) : json_show(CommonModel::$error_token, '账户为空');

    }

    //更改密码
    public static function updatePassword(array $data = []): Json
    {
        $rs = AccountModel::field('id,password,salt')
            ->where(['is_del' => CommonModel::$del_normal, 'id' => self::$aid])
            ->findOrEmpty()
            ->getData();//password,salt这两个字段在模型里定义了隐藏，所以要在这里使用getData方法获取原始数据

        if (empty($rs)) return json_show(CommonModel::$error_token, '该账户不存在');
        
        if (getPassword($data['old_password'], $rs['salt']) != $rs['password']) return json_show(CommonModel::$error_param, '密码错误');

        $salt = randomkeys(6);
        $password = getPassword($data['new_password'], $salt);

        $da = [
            'pwd' => $data['new_password'],
            'salt' => $salt,
            'password' => $password,
            'updaterid' => self::$aid,
            'updater' => self::$aname,
            'updatetime' => date('Y-m-d H:i:s'),
        ];

        $rs = AccountModel::where(['id' => self::$aid, 'is_del' => CommonModel::$del_normal])
            ->save($da);

        return $rs ? json_show(CommonModel::$success, '更改密码成功') : json_show(CommonModel::$error_param, '更改密码失败');

    }

    //视频列表
    public static function getVideoList(array $data = []): Json
    {
        $rs = AccountModel::field('id,video_ids')
            ->where(['id' => self::$aid, 'is_del' => CommonModel::$del_normal])
            ->findOrEmpty()
            ->toArray();

        if (empty($rs)) return json_show(CommonModel::$error_param, '该账户不存在');

        $db = VideoModel::where(['is_del' => CommonModel::$del_normal, 'status' => CommonModel::$status_normal])
            ->whereIn('id', $rs['video_ids']);

        $count = $db->count('id');

        $list = $db
            ->field('id,video_sn,video_name,video_url,video_img')
            ->page($data['page'], $data['size'])
            ->order(['weight' => 'desc', 'id' => 'desc'])
            ->select()
            ->toArray();

        return json_show(CommonModel::$success, '获取视频列表成功', ['count' => $count, 'list' => $list]);
    }

    //手机主题
    public static function theme(): Json
    {
        $group_id = GroupModel::where(['is_del' => CommonModel::$del_normal, 'company_id' => self::$company_id, 'card_id' => self::$card_id])
            ->value('id', 0);
        if (!$group_id) return json_show(CommonModel::$error_param, '该账户所对应的分组不存在');

        $rs = ThemeModel::field('id,code')
            ->where(['is_del' => CommonModel::$del_normal, 'group_id' => $group_id, 'status' => CommonModel::$status_normal])
            ->append(['modular'])
            ->withAttr('modular', function ($val, $data) {
                return Db::name('theme_modular')
                    ->field('id,title,type')
                    ->where(['is_del' => CommonModel::$del_normal, 'theme_id' => $data['id'], 'status' => CommonModel::$status_normal])
                    ->order(['addtime' => 'desc', 'id' => 'desc'])
                    ->append(['data'])
                    ->withAttr('data', function ($v, $d) {
                        return Db::name('theme_modular_data')
                            ->field('id,img,jump_type,jump_param,good_name,good_id,style_type')
                            ->where(['is_del' => CommonModel::$del_normal, 'theme_modular_id' => $d['id']])
                            ->order(['addtime' => 'desc', 'id' => 'desc'])
                            ->select()
                            ->toArray();
                    })
                    ->select()
                    ->toArray();
            })
            ->findOrEmpty()
            ->toArray();
        return empty($rs) ? json_show(CommonModel::$error_param, '该手机主题不存在') : json_show(CommonModel::$success, '获取手机主题成功', $rs);
    }


}